@@ -101,7 +101,7 @@ class MiniDNF():
         def emit(self, record):
             if record.getMessage():
                 if record.levelno > logging.WARNING:
-                        self._sink.error(record.getMessage())
+                    self._sink.error(record.getMessage())
                 elif record.levelno > logging.DEBUG:
                     self._sink.info(record.getMessage())
                 else:
@@ -166,6 +166,7 @@ class MiniDNF():
             dnf.callback.PKG_DOWNGRADE: _(
                 'Downgrading'
             ),
+            dnf.callback.PKG_REMOVE: _('Removing'),
             dnf.callback.PKG_INSTALL: _('Installing'),
             dnf.callback.PKG_OBSOLETE: _('Obsoleting'),
             dnf.callback.PKG_REINSTALL: _(
@@ -180,6 +181,7 @@ class MiniDNF():
         _FILEACTION_TRANSLATION = {
             dnf.callback.PKG_CLEANUP: _('Cleanup'),
             dnf.callback.PKG_DOWNGRADE: _('Downgraded'),
+            dnf.callback.PKG_REMOVE: _('Removed'),
             dnf.callback.PKG_INSTALL: _('Installed'),
             dnf.callback.PKG_OBSOLETE: _('Obsoleted'),
             dnf.callback.PKG_REINSTALL: _(
@@ -331,6 +333,12 @@ class MiniDNF():
         # its own transaction when doing a rollback.
         base.conf.clean_requirements_on_remove = False
 
+        # This causes DNF to either use the highest available version or fail.
+        # When it fails, it should show the reasons. Without this, it will
+        # simply ignore the unsatisfied dependencies of the highest version,
+        # and either install an older version or do nothing.
+        base.conf.best = True
+
         base.init_plugins(disabled_glob=self._disabledPlugins, cli=Cli(base))
 
         base.read_all_repos()
@@ -812,12 +820,9 @@ class MiniDNF():
         try:
             return [
                 {
-                    'operation': (
-                        'installed' if base._group_persistor.group(
-                            group.id
-                        ).installed
-                        else 'available'
-                    ),
+                    'operation': 'installed'
+                    if base.history.group.get(group.id) is not None
+                    else 'available',
                     'name': group.id,
                     'description': group.name,
                     'uservisible': group.visible
@@ -928,6 +933,7 @@ class Example():
         except IgnoreMe:
             pass
 
+
 if __name__ == '__main__':
     Example.main()
 
